@@ -3,7 +3,7 @@
 #include "intutil.h"
 #include "tokenizer.h"
 #include "stringutil.h"
-#include "vm.h"
+#include "liquid_vm.h"
 #include "variable.h"
 #include "context.h"
 #include "parse_context.h"
@@ -49,7 +49,7 @@ static void block_body_mark(void *ptr)
     } else {
         rb_gc_mark(body->as.intermediate.parse_context);
         if (body->as.intermediate.vm_assembler_pool)
-            vm_assembler_pool_gc_mark(body->as.intermediate.vm_assembler_pool);
+            rb_gc_mark(body->as.intermediate.vm_assembler_pool->self);
         if (body->as.intermediate.code)
             vm_assembler_gc_mark(body->as.intermediate.code);
     }
@@ -199,9 +199,14 @@ static tag_markup_t internal_block_body_parse(block_body_t *body, parse_context_
                 long name_len = name_end - name_start;
 
                 if (name_len == 0) {
-                    VALUE str = rb_enc_str_new(token.str_trimmed, token.len_trimmed, utf8_encoding);
-                    unknown_tag = (tag_markup_t) { str, str };
-                    goto loop_break;
+                    if (name_start < end && *name_start == '#') { // inline comment
+                        name_end++;
+                        name_len++;
+                    } else {
+                        VALUE str = rb_enc_str_new(token.str_trimmed, token.len_trimmed, utf8_encoding);
+                        unknown_tag = (tag_markup_t) { str, str };
+                        goto loop_break;
+                    }
                 }
 
                 if (name_len == 6 && strncmp(name_start, "liquid", 6) == 0) {
