@@ -67,28 +67,40 @@ static VALUE parse_number(parser_t *p)
     return out;
 }
 
+__attribute__((noreturn)) static void raise_invalid_expression_type(const char *expr, int expr_len)
+{
+    rb_enc_raise(utf8_encoding, cLiquidSyntaxError, "Invalid expression type '%.*s' in range expression", expr_len, expr);
+}
+
 static VALUE try_parse_constant_range(parser_t *p)
 {
     parser_t saved_state = *p;
 
     parser_must_consume(p, TOKEN_OPEN_ROUND);
 
+    const char *begin_str = p->cur.val;
     VALUE begin = try_parse_constant_expression(p);
+    const char *begin_str_end = p->cur.val;
     if (begin == Qundef) {
         *p = saved_state;
         return Qundef;
     }
     parser_must_consume(p, TOKEN_DOTDOT);
 
+    const char *end_str = p->cur.val;
     VALUE end = try_parse_constant_expression(p);
+    const char *end_str_end = p->cur.val;
     if (end == Qundef) {
         *p = saved_state;
         return Qundef;
     }
     parser_must_consume(p, TOKEN_CLOSE_ROUND);
 
-    begin = rb_funcall(begin, id_to_i, 0);
-    end = rb_funcall(end, id_to_i, 0);
+    begin = rb_check_funcall(begin, id_to_i, 0, NULL);
+    if (begin == Qundef) raise_invalid_expression_type(begin_str, (int)(begin_str_end - begin_str));
+
+    end = rb_check_funcall(end, id_to_i, 0, NULL);
+    if (end == Qundef) raise_invalid_expression_type(end_str, (int)(end_str_end - end_str));
 
     bool exclude_end = false;
     return rb_range_new(begin, end, exclude_end);
